; Remi Veilleux, 2010
; http://sites.google.com/site/rveilleux/coco3home
; This music synthesizer program mimics the sound of a Commodore 64 SID chip
; The music "COMMANDO.SID", by Rob Hubbard, has been hand converted to my CoCo synth-player music format
; Features:
; - 3 independant channels
; - Square, Sin, Sawtooth, Triangle, and White Noise waveforms
; - Supports Attack, Sustain, Release
; - 3 volume level
; - Glide and pitch bend
; - C64-style Arpeggio
; - 19 KHz realtime mixing quality sound output

; http://www.coco3.com/community/2010/01/can-a-coco-3-sound-like-a-c64/

; http://www.coco3.com/community/2010/02/cocosid2-playing-battle-valley/

; Note: With MESS, use  -wavwrite output.wav to output a wave for analysis


; This cut-down version (v0.04) for CoCo 1 - 3, and Dragon 32/64 by Steve Evans (Zephyr). 
; http://archive.worldofdragon.org/phpBB3/
; 8th March 2010

; Half rate (0.89 MHz) synth conversion and timing optimization by Ciaran Anscomb (sixxie).
; http://www.6809.org.uk/
; http://archive.worldofdragon.org/phpBB3/
; 12th Feb 2010


NOISE_FREQ_DOWN equ	40	; value that lowers NOISE waveform down to mimic C64 

			org	0
ssynth_freq		rmb	1 	; current note, doubled, used as 16-bit index into note array (69*2) should give 'A 440 Hz'
ssynth_wavetable	rmb	2	; currently active wavetable for synth
ssynth_note_wave	rmb	2	; selected note wavetable for synth
ssynth_wave_frag	rmb	1	; wavetable fragment index synth
ssynth_voice_ptr	rmb	2	; pointer into current playing pattern for synth
ssynth_song_ptr		rmb	2	; pointer into song (i.e.: to find the current pattern)
ssynth_note_duration	rmb	1	; current note duration for synth
ssynth_pause_duration	rmb	1	; current note pause duration for synth
ssynth_slur_note	rmb	1	; same inital value as pause_duration: used to check if next note will be 'slurred'
ssynth_pitch_alteration rmb	2	; pitch alteration: added to each note being played
ssynth_pitch_glide	rmb	2	; pitch glide speed: added to pitch alteration every frame
sizeof_ssynth		equ	*


	org $2a00
	
; jump to 'real startup' to allow definining procs before using them
start	bra start2

; variables...
tmp1		fcb	0
tmp2		fcb	0
tmp3		fcb	0

r4		fcb	12	; used for random generator..
r5		fcb	23
r6		fcb	34
r7		fcb 	45

modulate_dir	fcb	-1
modulate_index	fcb	255
backup_s	fdb	start-5

synth1		rmb	sizeof_ssynth
synth2		rmb	sizeof_ssynth
synth3		rmb	sizeof_ssynth

************************
* Main program start
start2	

	; initalize:
	orcc	#$50		; disable interrupts
	clra			; make sure dp reg = 0
	tfr	a,dp
	setdp	0

	ldx	<$74		; check if 64k mode operating system is active on dragon 64
	cmpx	#$7fff
	bls	set_stack
	rts

set_stack
	lds	#start-1	; set stack
	leax	1,s
	stx	<$9d
	ldx	<$72
	stx	,--s
	ldx	#patterns_end
	stx	<$76

	lda	$bfff
	ldx	$c000		; check for dos cart
	cmpx	#$444b
	bne	chk_coco3

	cmpa	#$b4		; check if machine is dragon or coco
	beq	dgndos
	clr	$ff40		; turn off coco disk motor
	bra	chk_coco3
dgndos	clr	$ff48		; turn off dragon disk motor

chk_coco3
	cmpa	#$1b		; check for coco 3
	bne	clrscn
	tst	<$e7		; check for default (width 32) text mode
	beq	clrscn
	pshs	a
	clr	<$e7		; set default text mode
	jsr	$e019
	puls	a

clrscn	bsr	cls		; clear text screen

	clrb			; select DAC as sound source and enable audio
	cmpa	#$b4
	beq	dgn_sndsel
	jsr	$a9a2
	jsr	$a976
	bra	set_dp_register
dgn_sndsel
	jsr	$baf1
	jsr	$bac5
	
set_dp_register
startdp	equ	start/256
	lda	#startdp	; set dp register
	tfr	a,dp
	setdp	startdp
	
	jsr	generate_waveform	
	
	; entry point for reset/restart song completely
reset_restart
	ldx	#brk_txt	; display break message
	bsr	shwtxt

	ldx	#$0000		
chk_break
	lda	#$fb		; check for break key during delay
	sta	$ff02
	lda	$ff00
	anda	#$40
	bne	rrloop
	clra			; reset dp
	tfr	a,dp
	lda	$bfff		; check if machine is dragon or coco
	cmpa	#$b4
	beq	dgn_audio_off	; disable audio and return to basic
	jmp	$a974
dgn_audio_off
	jmp	$bac3

rrloop	leax	-1,x		; delay loop
	bne	chk_break

	bsr	cls		; clear text screen

	bsr	init_song
	jmp	loop_sound	; start main sound loop
	
cls	ldx	#$0400
	stx	>$0088
	ldb	#$60
csloop	stb	,x+
	cmpx	#$05ff
	bls	csloop
	rts

shwtxt	clra
	tfr	a,dp
	sta	<$6f
txtloop	lda	,x+
	beq	resdp
	jsr	[$a002]
	bra	txtloop
resdp	ldd	#start
	tfr	a,dp
	rts

brk_txt	fcc	"PRESS <BREAK> NOW TO EXIT..."
	fcb	$0D,0

init_song
	; initialize song
	; Initialize start values for all synth
	ldd	#song_voice1
	std	synth1+ssynth_song_ptr
	ldd	#song_voice2
	std	synth2+ssynth_song_ptr
	ldd	#song_voice3
	std	synth3+ssynth_song_ptr
	clra
	clrb
	sta	synth1+ssynth_note_duration
	sta	synth1+ssynth_pause_duration
	std	synth1+ssynth_voice_ptr
	std	synth1+ssynth_pitch_alteration
	std	synth1+ssynth_pitch_glide

	sta	synth2+ssynth_note_duration
	sta	synth2+ssynth_pause_duration
	std	synth2+ssynth_voice_ptr
	std	synth2+ssynth_pitch_alteration
	std	synth2+ssynth_pitch_glide

	sta	synth3+ssynth_note_duration
	sta	synth3+ssynth_pause_duration
	std	synth3+ssynth_voice_ptr
	std	synth3+ssynth_pitch_alteration
	std	synth3+ssynth_pitch_glide

	lda	#1	; set 'no slur'
	sta	synth1+ssynth_slur_note
	sta	synth2+ssynth_slur_note
	sta	synth3+ssynth_slur_note

	ldd	#wavetable	; set all voice initially to 'mute' (wavetable entry 0)
	std	synth1+ssynth_wavetable
	std	synth2+ssynth_wavetable
	std	synth3+ssynth_wavetable
	rts

loop_sound
	jsr	process_song
	bsr	sin_wave2

	jsr	rect_modulator
	bsr	sin_wave2

	jsr	rect_modulator
	bsr	sin_wave2
	bra	loop_sound


sin_wave2
	; setup 3 synths frequency and waveform

	; Synth 1: final wave pointer in Y
	ldx	synth1+ssynth_wavetable
	lda	synth1+ssynth_wave_frag
	leax	a,x
	ldy	,x++	; read 16-bit wave pointer into y
	ldb	,x	; read pitch alteration into b
	adda	#4	; 4 bytes per entry, we have 16 entries
	sta	synth1+ssynth_wave_frag
	; setup final frequency
	ldx	#note_table
	addb	synth1+ssynth_freq	; 'b' contain pitch alteration: add base freq
	abx
	ldd	,x
	addd	synth1+ssynth_pitch_alteration	; add current pitch alteration
	lslb
	rola
	std	selfmod_addd_freq1+1	; store into self modifying code the final frequency
	ldd	synth1+ssynth_pitch_alteration	; process pitch alteration glide
	addd	synth1+ssynth_pitch_glide
	std	synth1+ssynth_pitch_alteration

	; Synth 2: final wave pointer in U
	ldx	synth2+ssynth_wavetable
	lda	synth2+ssynth_wave_frag
	leax	a,x
	ldu	,x++	; read 16-bit wave pointer into u
	ldb	,x	; read pitch alteration into b
	adda	#4	; 4 bytes per entry, we have 16 entries
	sta	synth2+ssynth_wave_frag
	; setup final frequency
	ldx	#note_table
	addb	synth2+ssynth_freq	; 'b' contain pitch alteration: add base freq
	abx
	ldd	,x
	lslb
	rola
	std	selfmod_addd_freq2+1	; store into self modifying code the final frequency

	; Synth 3: final wave pointer in S
	ldx	synth3+ssynth_wavetable
	lda	synth3+ssynth_wave_frag
	leax	a,x
	lds	,x++	; read 16-bit wave pointer into s
	ldb	,x	; read pitch alteration into b
	adda	#4	; 4 bytes per entry, we have 16 entries
	sta	synth3+ssynth_wave_frag
	; setup final frequency
	ldx	#note_table
	addb	synth3+ssynth_freq	; 'b' contain pitch alteration: add base freq
	abx
	ldd	,x
	lslb
	rola
	std	selfmod_addd_freq3+1	; store into self modifying code the final frequency

synthdp	equ	first_selfmod/256
	lda	#synthdp
	tfr	a,dp

	; loop for 205 iteration (to output 50Hz)
	; 205 -> 0 + 205
	ldx	#219
synth_mixer
	; First synth:
first_selfmod
selfmod_ldd_freq1
	ldd	#$0000			; 3
selfmod_addd_freq1
	addd	#$0000			; 4
	std	selfmod_ldd_freq1+1	; 6 (5 if direct)
	ldb	a,y			; 5
	stb	selfmod_addb_sample1+1	; 5 (4 if direct)
	; Total for synth 1: 23 clocks

	; Second synth:
selfmod_ldd_freq2
	ldd	#$0000			; 3
selfmod_addd_freq2
	addd	#$0000			; 4
	std	selfmod_ldd_freq2+1	; 6 (5 if direct)
	ldb	a,u			; 5
	stb	selfmod_addb_sample2+1	; 5 (4 if direct)
	; Total for synth 2: 23 clocks

	; Third synth:
selfmod_ldd_freq3
	ldd	#$0000			; 3
selfmod_addd_freq3
	addd	#$0000			; 4
	std	selfmod_ldd_freq3+1	; 6 (5 if direct)
	ldb	a,s			; 5
	; Total for synth 3: 18 clocks

selfmod_addb_sample1
	addb	#$00			; 2
selfmod_addb_sample2
	addb	#$00			; 2
	;andb	#$fc
	stb	$ff20			; 5
	; Total for mixing + inner loop: 18

	leax	-1,x			; 5
	bne 	synth_mixer		; 3

	; Total for 1 sample loop iteration for 3 channel: (we are not accounting for outer loop, insignificant)
	; 82 clocks (77 clocks)  now 76!  need to recalc all this:
	; Effective sample rate:
	; 1789772 / 82 = 21826 Hz (slow: 10913 Hz) (direct: 23243.8/11621.9)
	; To have a C64 Pal frequency of 50Hz, we need to play:
	; 21826 / 50 Hz = 436 iteration (which is 256 + 180 loop)
	; slow: 10913 / 50 Hz = 218 iterations
	; direct: 465/232
	; For a waveform of 256 sample played at 'freq 1.0' (i.e.: freq=256), we have:
	; 21826 / 256 = 85.26 Hz
	; direct: 23243.8 / 256 = 90.796 Hz

	; Speed:
	; We play 361 samples x 4 batch = 1444 samples
	; 18080 / 1444 = 12.5 note per second
	; 12.5 * 60 = 750 note per minute
	; 750 / 8 = 93.75 bpm, using 1/8th note duration

	; C64 Commando plays at 125 BPM
	; So: 125 * 8 = 1000 / 60 = 16.7 note per second
	; 19456 / 16.7 = 1165 samples / 389 = exactly 3 batchs

	lda	#startdp
	tfr	a,dp

	lds	backup_s
	rts

*******************
	
note_continue
	; while a note is playing, this channel does nothing else
	ldb	ssynth_wave_frag,u
	deca
	sta	ssynth_note_duration,u
	; when we reach note duration 0, it's time to play 'release' part of wave_frag
	beq	note_continue_release
	; wave_frag needs to wrap around from 12 to 48 while the note plays
	lda	#48
	cmpa	ssynth_wave_frag,u
	bne	note_continue_sustain_done
	lda	#12
	sta	ssynth_wave_frag,u
note_continue_sustain_done
	rts
note_continue_release
	; 'release' starts at fragment offset 48
	; slur check if 'slur_note' is zero for this note, we DON'T play the 'release' part
	; in order to 'slur' note
	lda	ssynth_slur_note,u
	bne	note_continue_release_accept
	lda	#12	; set fragment to offset start of 'sustain' part: this means we are not doing the release
	bra	note_continue_release_cont
note_continue_release_accept
	lda	#48	; ok, jump to 'release' portion
note_continue_release_cont
	sta	ssynth_wave_frag,u
	rts

pause_continue
	; while a note is pausing, this channel does nothing else
	deca
	sta	ssynth_pause_duration,u
	ldd	#wavetable
	std	ssynth_wavetable,u	; set silence wavetable #00 as active
	clr	ssynth_wave_frag,u	; leave 'wave_frag' stuck at fragment 0
	rts

**********************
; process_song
process_song
	; process each voice
	ldu	#synth1
	bsr	process_voice

	ldu	#synth2
	bsr	process_voice

	ldu	#synth3
	; optimisation: don't need to 'jsr / rts', process_voice will 'rts' itself

process_voice
	; First, check if we need to fetch a new pattern: (i.e.: when our voice_ptr is 0000)
	ldd	ssynth_voice_ptr,u
	bne 	process_voice_cont
	; Fetch a pattern from song ptr and move song_ptr to next
new_pattern_request
	ldx	ssynth_song_ptr,u
	ldb	,x+
	; if pattern is 00, we have reached end of pattern data for this track
	beq	new_pattern_end
	; if pattern is 255 (-1), special case: instant jump to complete reset/restart
	cmpb	#255
	bne	new_pattern_ok
	leas	4,s			; remove return addresses from stack
	jmp	reset_restart
new_pattern_end
	clr	ssynth_wave_frag,u	; leave 'wave_frag' stuck at fragment 0 forever
	rts
new_pattern_ok
	; update song ptr
	stx	ssynth_song_ptr,u
	; now compute pattern pointer location
	ldx	#pattern_ptr-2
	lslb	; each pattern ptr is 16-bit so multiply by 2 to get 16-bit offset
	abx
	ldx	,x
	stx	ssynth_voice_ptr,u

process_voice_cont
	; Now check is a note is currently in progress (if note_duration is not 0)
	lda	ssynth_note_duration,u
	bne	note_continue
	; ..and check is a silence/pause is in progress (if pause_duration is not 0)
	lda	ssynth_pause_duration,u
	bne	pause_continue

	; Process commands from song voice data:
	ldx	ssynth_voice_ptr,u
	ldb	,x+	; fetch command number
	lslb
	clra
	addd	#cmd_jumptable
	tfr	d,pc

cmd_jumptable
	bra	command_00
	bra	command_01
	bra	command_02
	bra	command_03
	bra	command_04
	bra	command_05
	bra	command_06
	bra	command_07
	bra	command_08

command_done
	stx	ssynth_voice_ptr,u	; update voice pointer
	bra	process_voice	; reloop: until we hit a play note command

command_00
	; End of pattern: go to new pattern request	 
	bra	new_pattern_request
command_01
	lda	#1
	bra	command_done
command_02
	pshs	u	; WARNING: 'u' synth pointer is not used and invalided in this command
	; Wavetable define:
	lda	#60	; 60 bytes per entry
	ldb	,x+	; fetch wavetable index
	mul
	addd	#wavetable	; set destination
	tfr	d,y
	lda	#15	; copy 15 pointers
	sta	tmp1
wavecpy	ldd	,x++	; read 'a=wave index', 'b=pitch alteration'
	cmpa	#16	; Special case: check if this is a 'noise' waveform (i.e.: if wave >= 16)
	blt	L0
	; for noise waveform, we lower the pitch to mimic the C64 low-frequency white noise generator
	subb	#NOISE_FREQ_DOWN	; lower pitch several note down
L0	lsla
	ldu	#waveform_lookup
	ldu	a,u
	stu	,y++
	aslb	; pre-multiply by 2 for adequate note shift (signed 8-bit)
	stb	,y++	; (note: 1 byte is wasted here for 32-bit alignment purpose)
	dec	tmp1
	bne	wavecpy
	puls	u

	bra	command_done
command_03
	; play note
	ldd	,x++	; a=note, b=length
	lsla
	sta	ssynth_freq,u
	decb		; pre-decrement duration by 1 since this loop will play '1' duration
	stb	ssynth_note_duration,u

	lda	ssynth_slur_note,u	; check if previous note as a 'slur_note' indication
	beq	play_note_slurred	; yes: slur!
	clra				; no: regular play: start fragment at 0
	sta	ssynth_wave_frag,u	; set fragment when starting a new note
	bra	play_note_cont
play_note_slurred
	; Unneeded: just keep fragment where it was: it will continue and wrap around as normal
	;lda	#12			; set starting frag to skip 'attack' since we are slurring
play_note_cont

	lda	,x+	; a=pause duration
	sta	ssynth_pause_duration,u
	sta	ssynth_slur_note,u	; set both pause and slur as the same initial value
	stx	ssynth_voice_ptr,u
	ldd	ssynth_note_wave,u
	std	ssynth_wavetable,u	; set note wavetable as active

	; release mod:
	; if pause duration is not 0, then increase note length by 1 and decrease pause length by 1
	; so the 'release' part will play in the first time of 'pause'
	; this should make note longer and feel more 'attached' than the harsh mute pause
	lda	ssynth_pause_duration,u
	beq	play_note_release_mod_skip
	deca
	sta	ssynth_pause_duration,u
	inc	ssynth_note_duration,u
play_note_release_mod_skip
	rts
command_04
	; unused
command_05
	; unused
command_06
	; Set note wavetable (which will be used for future playing notes)
	lda	#60	; 60 bytes per entry
	ldb	,x+	; fetch wavetable index
	mul
	addd	#wavetable
	std	ssynth_note_wave,u
	bra	command_done
command_07
	; Set pitch alteration
	leax	1,x	; skip empty byte since command is fdb 16-bit
	ldd	,x++
	std	ssynth_pitch_alteration,u
	bra	command_done
command_08
	; Set Set pitch-glide speed
	leax	1,x	; skip empty byte since command is fdb 16-bit
	ldd	,x++
	std	ssynth_pitch_glide,u
	bra	command_done

;***************
; modulate the rect waveform over time: simulate a 'LFO' modulation
; Note: only 'rect256' 100% volume is modulated
; not 50% and 25% volume
rect_modulator
	ldx	#rect256
	ldb	modulate_index
	abx
	lda	modulate_dir
	beq	rect_mod_dir_0
rect_mod_dir_1
	lda	#-36+42
	sta	,x		; modulate two byte per 'frame'
	sta	,-x
	sta	,-x
	sta	,-x
	subb	#4
	stb	modulate_index
	cmpb	#127
	bne	rect_done	; once we reach '128', toggle value
	lda	#0
	ldb	#128		; modulate samples 128 to 255
	bra	rect_toggle
rect_mod_dir_0
	lda	#36+42
	sta	,x+		; modulate two byte per 'frame'
	sta	,x+		;
	sta	,x+		;
	sta	,x
	addb	#4
	stb	modulate_index
	bne	rect_done	; once we reach '0', toggle value
	lda	#-1
	ldb	#255		; modulate samples 128 to 255
rect_toggle
	stb	modulate_index
	sta	modulate_dir
rect_done
	rts

******************** generate waveforms:
generate_waveform
; generate square:
	ldx	#square256
	lda	#224	; 224 / 32 is a pulse wave,   128 / 128 would be a perfect square wave
			; since only track 1 in Battle Valley uses square wave
	ldb	#85	; note: using values over maximum range 0..84
L1	stb	,x+
	deca
	bne	L1
	lda	#32
	ldb	#0
L2	stb	,x+
	deca	
	bne	L2
	
; generate rect:
	ldx	#rect256
	lda	#88	; 224 / 32 is a pulse wave,   128 / 128 would be a perfect square wave
	ldb	#0	; inverted of square for better mixing
L3	stb	,x+
	deca
	bne	L3
	lda	#(256-88)
	ldb	#84
L4	stb	,x+
	deca	
	bne	L4
	
;generate mute:
	ldx	#mute256
	lda	#0	; loop 256 times, 
	ldb	#42	; fill with '42'
L5	stb	,x+
	deca
	bne	L5
	
;generate sawtooth
	ldx	#saw256
	lda	#85	; loop 85 times
	ldb	#0	; start at 0
	stb	,x+
L6	stb	,x+
	stb	,x+
	stb	,x+
	incb
	deca
	bne	L6

;generate triangle
	ldx	#triangle256
	lda	#42	; loop 42 times
	ldb	#0	; start at 0
	stb	,x+
L7	stb	,x+
	incb
	stb	,x+
	stb	,x+
	incb
	deca
	bne	L7
	stb	,x
	ldx	#triangle256
	lda	#128	; loop 128 times (we only 'mirror' half the triangle)
L8	ldb	#85
	subb	,x+
	stb	127,x
	deca
	bne	L8
	
; generate sin
	ldx	#triangle256
	ldy	#sin256
	lda	#0	; loop 256 times: sin is a copy of triangle to save some memory
L9	ldb	,x+
	stb	,y+
	deca
	bne	L9

; generate noise	
	ldx	#noise2048
noise_gen_loop
; 31-bit random number 
; primitive polynomial order 31 
; (1)0100000 10101010 10010101 10110101 (20 AA 95 B5) 
; shift before XOR

rand_31
   	lsl   r4         ; shift first 
   	rol   r5
   	rol   r6 
   	rol   r7
   	ldb	r7
   	andb	#%10000000 ; test MSB 
   	beq	rand_done            ; clear, no XOR
   	ldb	r4
   	eorb	#$B5	; first poly byte XOR it in
   	stb	r4
   	ldb	r5
   	eorb 	#$95    ; second byte of poly
   	stb	r5
   	ldb	r6
   	eorb	#$AA      ; same routine
   	stb	r6
   	ldb	r7
   	eorb	#$20
   	stb	r7 
rand_done
   	ldb	r7
   	eorb	r6
   	eorb	r5
   	eorb	r4

	lsrb	; try to keep 'b' in valid range [0..85]
	cmpb	#85
	bhi	rand_31	; recompute a new random
noise_continue
	stb	,x+
	cmpx	#noise2048+2048
	bne	noise_gen_loop
	
; fade 510 last entries in noise	
	ldx	#noise2048+2048-510
	lda	#0
	sta	tmp1	; tmp1 is recenter offset
	lda	#6
	sta	tmp2	; tmp2 is counter before moving recenter
	; each entry is reduced progressively to 0 but recentered at 42
	lda	#255
	sta	tmp3	; tmp3 is current attenuation value
noise_fade_loop
	lda	tmp3
	ldb	,x
	mul
	adda	tmp1
	sta	,x+

	lda	tmp3
	ldb	,x
	mul
	adda	tmp1
	sta	,x+
	dec	tmp2	; keep track of recentering: goes from 0 to 42, inc by 1 every 6 iteration
	bne	noise_fade_cont
	lda	#6
	sta	tmp2
	inc	tmp1
noise_fade_cont
	dec	tmp3
	bne	noise_fade_loop

	; generate 50% volume and 25% volume
	ldx	#square256
	ldy	#square256_s1
	bsr	waveform_volume_half
	ldx	#square256_s1
	ldy	#square256_s2
	bsr	waveform_volume_half

	ldx	#saw256
	ldy	#saw256_s1
	bsr	waveform_volume_half
	ldx	#saw256_s1
	ldy	#saw256_s2
	bsr	waveform_volume_half

	ldx	#triangle256
	ldy	#triangle256_s1
	bsr	waveform_volume_half
	ldx	#triangle256_s1
	ldy	#triangle256_s2
	bsr	waveform_volume_half

	ldx	#sin256
	ldy	#sin256_s1
	bsr	waveform_volume_half
	ldx	#sin256_s1
	ldy	#sin256_s2
	bsr	waveform_volume_half

	ldx	#rect256
	ldy	#rect256_s1
	bsr	waveform_volume_half
	ldx	#rect256_s1
	ldy	#rect256_s2
	bsr	waveform_volume_half

	rts

********************** waveform_volume_half
* Half the volume of a 256 sample fragment into a destination area
* input:
* 'x': points to source sample buffer
* 'y': points to destination sample buffer
waveform_volume_half
	clra
L10	ldb	,x+
	asrb		; divide waveform by 2
	addb	#21	; re-center waveform (needed because waveform is unsigned centered at 42)
	stb	,y+
	deca
	bne	L10
	rts

waveform_lookup
	fdb	mute256+128		; 0  Silence
	fdb	sin256+128		; 1  Sin volume 100%
	fdb	saw256+128		; 2  Saw volume 100%
	fdb	triangle256+128		; 3  Tri volume 100%
	fdb	square256+128		; 4  Sqr volume 100%
	fdb	sin256_s1+128		; 5  Sin volume 50%
	fdb	saw256_s1+128		; 6  Saw volume 50%
	fdb	triangle256_s1+128	; 7  Tri volume 50%
	fdb	square256_s1+128	; 8  Sqr volume 50%
	fdb	sin256_s2+128		; 9  Sin volume 25%
	fdb	saw256_s2+128		; 10 Saw volume 25%
	fdb	triangle256_s2+128	; 11 Tri volume 25%
	fdb	square256_s2+128	; 12 Sqr volume 25%
	fdb	rect256+128		; 13 Rect inverted modulated pulse		
	fdb	rect256_s1+128		; 14 Rect 50% inverted modulated pulse		
	fdb	rect256_s2+128		; 15 Rect 25% inverted modulated pulse		
	
	fdb	noise2048+128		; 16 Noise: waveform 16..23 will have their frequency lowered to 'match'
	fdb	noise2048+256+128	; 17        the C64 white noise which seems to play a lot lower (by NOISE_FREQ_DOWN)
	fdb	noise2048+512+128	; 18
	fdb	noise2048+768+128	; 19
	fdb	noise2048+1024+128	; 20
	fdb	noise2048+1280+128	; 21
	fdb	noise2048+1536+128	; 22
	fdb	noise2048+1792+128	; 23

; Note table based on frequency of 80.36 Hz
; Note number corresponds to midi notes: http://www.phys.unsw.edu.au/jw/notes.html
; 0..127 notes. 'A 440 Hz' is note 69
; Special case: note 0 is 00 freq = silence
; about note 39 or 40 is 1:1 play ratio (freq=256)

note_table
	fdb 23,24,26,27,29,30,32,34,36
	fdb 38,41,43,46,48,51,54,57,61,64,68,72
	fdb 77,81,86,91,96,102,108,115,121,129,136,144
	fdb 153,162,172,182,193,204,216,229,243,257,273,289
	fdb 306,324,344,364,386,409,433,459,486,515,545,578
	fdb 612,649,687,728,771,817,866,917,972,1030,1091,1156
	fdb 1224,1297,1374,1456,1543,1634,1732,1835,1944,2059,2182,2312
	fdb 2449,2595,2749,2912,3086,3269,3463,3669,3888,4119,4364,4623
	fdb 4898,5189,5498,5825,6171,6538,6927,7339,7775,8237,8727,9246
	fdb 9796,10378,10995,11649,12342,13076,13853,14677,15550,16475,17454,18492
	fdb 19592,20757,21991,23299,24684,26152,27707,29355,31100,32949,34908


	; '1' = 3 frame, a pattern is 32 notes * 3 = 96 frames per pattern
	; (double pattern are 192 frames long)
	; Total Commando song: 3:56.12 (i.e.: 11806 frames / 96 = 123 patterns)

song_voice1
	; pattern list for this track:
	fcb	01	; init instrument: don't SKIP THIS PATTERN!

	fcb	05,05	; frame 1,193
	fcb	05,05	; frame 385,577
	fcb	08,09	; frame 769,961
	fcb	10,11	; frame 1153,1345
	fcb	08,09	; frame 1537,1729 (repeat)
	fcb	10,11	; frame 1921,2113
	fcb	17,18	; frame 2305,2497
	fcb	19,20	; frame 2689,2881
	fcb	19,20	; frame 3073,3265 (repeat)
	fcb	21,21	; frame 3457,3649
	fcb	22,23	; frame 3841,4033
	fcb	22,23	; frame 4225,4417 (repeat)
	fcb	24,24	; frame 4609,4801
	fcb	25,26	; frame 4993,5185
	fcb	27,27,27,27	; frame 5377,5569
	fcb	27,27,27,27	; frame 5761,5953
	fcb	21,21	; frame 6145,6337
	fcb	27,27,27,27	; frame 6529,6721
	fcb	21,27,27	; frame 6913,7105
	fcb	28,29	; frame 7297,7489
	fcb	30,31	; frame 7681,7873
	fcb	32,33	; frame 8065,8257
	fcb	34,35	; frame 8449,8641
	fcb	36,36	; frame 8833,9025
	fcb	37,37	; frame 9217,9409
	fcb	38,38	; frame 9601,9793
	fcb	22,23	; frame 9985,10177
	fcb	27,27,39	; frame 10369,10561
	fcb	21,27	; frame 10657,10849
	fcb	24,27	; frame 10945,11137
	fcb	25,26	; frame 11233,11425
	fcb	27,39	; frame 11617,11713

	fcb	255	; reset/restart song indefinitely
	fcb	00

song_voice2
	; pattern list for this track:
	fcb	06,07	; frame 1,193
	fcb	06,07	; frame 385,577
	fcb	06,07	; frame 769,961
	fcb	15,16	; frame 1153,1345
	fcb	06,07	; frame 1537,1729 (repeat)
	fcb	15,16	; frame 1921,2113
	fcb	06,07	; frame 2305,2497
	fcb	06,07	; frame 2689,2881
	fcb	06,07	; frame 3073,3265
	fcb	45,45	; frame 3457,3649
	fcb	46,46	; frame 3841,4033
	fcb	46,46	; frame 4225,4417
	fcb	47,47	; frame 4609,4801
	fcb	48,48	; frame 4993,5185
	fcb	49,49,49,49	; frame 5377,5569
	fcb	49,49,49,49	; frame 5761,5953
	fcb	45,45	; frame 6145,6337
	fcb	49,49,49,49	; frame 6529,6721
	fcb	45,49,49	; frame 6913,7105
	fcb	45,45	; frame 7297,7489
	fcb	45,45	; frame 7681,7873
	fcb	45,45	; frame 8065,8257
	fcb	45,45	; frame 8449,8641
	fcb	45,45	; frame 8833,9025
	fcb	46,46	; frame 9217,9409
	fcb	46,46	; frame 9601,9793
	fcb	46,46	; frame 9985,10177
	fcb	49,49,50	; frame 10369,10561
	fcb	45,49	; frame 10657,10849
	fcb	47,49	; frame 10945,11137
	fcb	48,48	; frame 11233,11425
	fcb	49,50	; frame 11617,11713

	fcb	00


song_voice3
	; pattern list for this track:
	fcb	02,02,03,04	; frame 1,97,193,289
	fcb	02,02,03,04	; frame 385,481,577,673
	fcb	02,02,03,04	; frame 769,865,961,1057
	fcb	12,12,13,14	; frame 1153,1249,1345,1441
	fcb	02,02,03,04	; frame 1537,1729 (repeat)
	fcb	12,12,13,14	; frame 1921,2113
	fcb	02,40,41,02	; frame 2305,2401,2497,2593
	fcb	02,02,03,04	; frame 2689,2881
	fcb	02,02,03,04	; frame 3073,3265
	fcb	02,02,40,40	; frame 3457,3649
	fcb	42,42,13,13	; frame 3841,4033
	fcb	42,42,13,13	; frame 4225,4417
	fcb	40,40,40,40	; frame 4609,4801
	fcb	41,41,41,41	; frame 4993,5185
	fcb	43,43,43,43	; frame 5377,5569
	fcb	43,43,43,43	; frame 5761,5953
	fcb	02,02,40,40	; frame 6145,6337
	fcb	43,43,43,43	; frame 6529,6721
	fcb	02,40,43,43	; frame 6913,7105
	fcb	02,02,40,40	; frame 7297,7489
	fcb	02,02,40,40	; frame 7681,7873
	fcb	02,02,40,40	; frame 8065,8257
	fcb	02,02,40,40	; frame 8449,8641
	fcb	02,02,40,40	; frame 8833,9025
	fcb	42,42,13,13	; frame 9217,9409
	fcb	42,42,13,13	; frame 9601,9793
	fcb	42,42,13,13	; frame 9985,10177
	fcb	43,43,44	; frame 10369,10561
	fcb	02,40,43	; frame 10657,10849
	fcb	40,40,43	; frame 10945,11137
	fcb	41,41,41,41	; frame 11233,11425
	fcb	43,44		; frame 11617,11713
	fcb	00

pattern_ptr
	fdb	pattern_01
	fdb	pattern_02
	fdb	pattern_03
	fdb	pattern_04
	fdb	pattern_05
	fdb	pattern_06
	fdb	pattern_07
	fdb	pattern_08
	fdb	pattern_09
	fdb	pattern_10
	fdb	pattern_11
	fdb	pattern_12
	fdb	pattern_13
	fdb	pattern_14
	fdb	pattern_15
	fdb	pattern_16
	fdb	pattern_17
	fdb	pattern_18
	fdb	pattern_19
	fdb	pattern_20
	fdb	pattern_21
	fdb	pattern_22
	fdb	pattern_23
	fdb	pattern_24
	fdb	pattern_25
	fdb	pattern_26
	fdb	pattern_27
	fdb	pattern_28
	fdb	pattern_29
	fdb	pattern_30
	fdb	pattern_31
	fdb	pattern_32
	fdb	pattern_33
	fdb	pattern_34
	fdb	pattern_35
	fdb	pattern_36
	fdb	pattern_37
	fdb	pattern_38
	fdb	pattern_39
	fdb	pattern_40
	fdb	pattern_41
	fdb	pattern_42
	fdb	pattern_43
	fdb	pattern_44
	fdb	pattern_45
	fdb	pattern_46
	fdb	pattern_47
	fdb	pattern_48
	fdb	pattern_49
	fdb	pattern_50

; this is our special init instrument pattern: no note played, total duration: 0
pattern_01
	; a wavetable is defined using 5 group of 3 'wave / pitch alteration'
	; first group (0..2): attack portion
	; second group (3..11): sustain portion (loops around if sound is long)
	; third group (12..14): release portion, always played at the end
	;   (unless the sound duration is 1: in this case only the 'attack' portion plays)

	fcb	02,00,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0 ; define wavetable 00 : complete silence
	fcb	02,01,4,0,4,0,4,0,4,0,4,0,4,0,4,0,4,0,4,0,4,0,4,0,4,0,4,0,8,0,12,0 ; define wavetable 01 : voice 3 bass square
	fcb	02,02,16,0,17,0,18,12,10,0,20,12,21,0,10,0,20,12,21,0,0,0,0,0,0,0,21,0,22,0,23,0 ; define wavetable 02 : snare drum
	fcb	02,03,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0 ; define wave 3: plain sin (unused)
	fcb	02,04,2,0,2,0,2,0,2,0,2,0,2,0,2,0,2,0,2,0,2,0,2,0,2,0,2,0,2,0,2,0 ; define wave 4: plain sawtooth (unused)
	fcb	02,05,3,0,3,0,3,0,3,0,3,0,3,0,3,0,3,0,3,0,3,0,3,0,3,0,3,0,3,0,3,0 ; define wave 5: plain triangle (unused)

	fcb	02,06,3,0,16,0,17,-55,3,0,3,-70,3,0,3,-70,3,0,3,-70,0,0,0,0,0,0,7,0,11,-70,0,0 ; define wave 6: voice 1 first commando note
	fcb	02,07,3,0,16,0,17,12,3,0,3,12,3,0,3,12,3,0,3,12,0,0,0,0,0,0,7,0,11,12,0,0 ; define wave 7: voice 1 second sound
	fcb	02,08,4,0,16,0,17,12,4,0,4,12,4,0,4,12,4,0,4,12,4,0,4,12,4,0,4,12,8,0,12,12 ; define wave 8: voice 2 main instrument
	fcb	02,09,3,0,16,0,17,-41,3,0,3,-41,3,0,3,-41,3,0,3,-41,0,0,0,0,0,0,7,0,11,-41,0,0 ; define wave 9:

	fcb	02,10,4,0,19,0,21,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0 ; voice 2 'little tic tic tic'

	fcb	02,11,15,0,14,0,13,0,13,0,13,0,13,0,13,0,13,0,13,0,13,0,13,0,13,0,13,0,14,0,15,0 ; voice 1 'song main instrument'

	fcb	02,12,4,0,16,0,17,0,4,-1,4,-3,4,-4,4,-6,4,-8,4,-10,4,-12,4,-15,4,-18,4,-22,4,-28,4,-35 ; voice 1 'tom drum'
	fcb	02,13,4,0,16,0,17,0,4,0,4,-1,4,-2,0,0,0,0,0,0,0,0,0,0,0,0,4,-3,4,-5,4,-6 ; voice 1 'mini tom drum'
	fcb	02,14,4,0,16,0,17,-20,4,0,4,-51,4,0,4,-51,4,0,4,-51,4,0,4,-51,4,0,4,-51,4,0,4,-51 ; voice 1 'end song tom drum'

	fcb	02,15,3,0,16,0,17,-20,3,0,4,-51,3,0,4,-51,3,0,4,-51,3,0,4,-51,3,0,4,-51,3,0,4,-51 ; voice 2 special drum 1
	fcb	02,16,3,0,16,0,17,0,3,0,3,12,3,0,3,12,3,0,3,12,3,0,3,12,3,0,3,12,3,0,3,12 ; voice 2 special drum 2

	; test stuff
	;fcb	06,11
	;fcb	03,60,255,1

	fcb	00	; End of pattern

pattern_02
	; Voice 3: 'A-1' measure
	fcb	06,01	; activate wavetable
	fcb	03,33,5,1 ; play notes..
	fcb	03,45,1,1

	fcb	06,02	; activate wavetable
	fcb	03,58,3,1

	fcb	06,01	; activate wavetable
	fcb	03,33,3,1 ; 4 -> 3 + 1 pause
	fcb	03,33,7,1

	fcb	06,02	; activate wavetable
	fcb	03,58,3,1

	fcb	06,01	; activate wavetable
	fcb	03,43,1,1
	fcb	03,45,1,1
	fcb	00	; End of pattern

pattern_03
	; Voice 3: 'A#1' measure
	fcb	06,01	; activate wavetable
	fcb	03,34,5,1 ; play notes..
	fcb	03,46,1,1

	fcb	06,02	; activate wavetable
	fcb	03,58,3,1

	fcb	06,01	; activate wavetable
	fcb	03,34,3,1
	fcb	03,34,7,1

	fcb	06,02	; activate wavetable
	fcb	03,58,3,1

	fcb	06,01	; activate wavetable
	fcb	03,45,1,1
	fcb	03,46,1,1
	fcb	00	; End of pattern

pattern_04
	; Voice 3: 'E-1' measure
	fcb	06,01	; activate wavetable
	fcb	03,28,5,1 ; play notes..
	fcb	03,40,1,1

	fcb	06,02	; activate wavetable
	fcb	03,58,3,1

	fcb	06,01	; activate wavetable
	fcb	03,28,3,1
	fcb	03,28,7,1

	fcb	06,02	; activate wavetable
	fcb	03,58,3,1

	fcb	06,01	; activate wavetable
	fcb	03,38,1,1
	fcb	03,40,1,1
	fcb	00	; End of pattern

pattern_05
	; Voice 1: first pattern (double)
	fcb	06,06
	fcb	03,100,3,1
	fcb	06,07
	fcb	03,93,3,1

	fcb	06,08
	fcb	03,69,3,1
	fcb	03,69,3,1
	fcb	03,69,5,1
	fcb	03,69,5,1
	fcb	03,67,5,1
	fcb	03,69,1,1
	fcb	03,69,3,1
	fcb	03,69,3,1
	fcb	03,67,3,1
	fcb	03,69,1,1
	fcb	03,67,1,1
	fcb	03,69,3,1

	fcb	06,09
	fcb	03,100,3,1

	fcb	06,07
	fcb	03,93,3,1
	fcb	00	; End of pattern

pattern_06
	; Voice 2: first pattern (double)
	fcb	06,10
	fcb	03,71,1,1
	fcb	03,83,1,1
	fcb	03,83,1,1
	fcb	03,83,1,1

	fcb	06,08
	fcb	03,64,3,1
	fcb	03,64,3,1
	fcb	03,65,5,1
	fcb	03,64,5,1
	fcb	03,62,3,1

	fcb	06,10
	fcb	03,83,1,1
	fcb	03,83,1,1
	fcb	03,83,1,1
	fcb	03,83,1,1

	fcb	06,08
	fcb	03,64,3,1
	fcb	03,64,3,1
	fcb	03,64,7,9
	fcb	00	; End of pattern

pattern_07
	; Voice 2: second pattern (double)
	fcb	06,10
	fcb	03,83,1,1
	fcb	03,83,1,1
	fcb	03,83,1,1
	fcb	03,83,1,1

	fcb	06,08
	fcb	03,64,3,1
	fcb	03,64,3,1
	fcb	03,65,5,1
	fcb	03,64,5,1
	fcb	03,62,3,3

	fcb	03,64,1,1
	fcb	03,64,3,1
	fcb	03,64,3,1
	fcb	03,64,3,1
	fcb	03,64,7,9
	fcb	00	; End of pattern

pattern_08
	; Voice 1: refrain music first pattern normal in A-4 (69) (double)
	fcb	06,02
	fcb	03,62,1,1

	fcb	06,11
	fcb	03,69,1,1
	fcb	03,69,3,1
	fcb	03,69,3,1
	fcb	03,69,3,1
	fcb	03,69,7,1
	fcb	03,69,5,1
	fcb	03,69,3,1
	fcb	03,76,1,1
	fcb	03,76,3,1
	fcb	03,76,3,1
	fcb	03,76,3,1
	fcb	03,76,7,1
	fcb	06,12
	fcb	03,56,4,4
	fcb	00	; End of pattern

pattern_09
	; Voice 1: refrain music second pattern normal (double)
	fcb	06,11
	fcb	03,77,7,1
	fcb	03,76,7,1
	fcb	03,77,7,1
	fcb	03,76,7,3

	fcb	03,71,1,1
	fcb	03,71,3,1
	fcb	03,71,3,1
	fcb	03,71,3,1
	fcb	03,71,7,1

	fcb	06,12
	fcb	03,56,4,4
	fcb	00	; End of pattern

pattern_10
	; Voice 1: refrain music first pattern raised to C-5 (72) (double)
	fcb	06,02
	fcb	03,62,1,1

	fcb	06,11
	fcb	03,72,1,1
	fcb	03,72,3,1
	fcb	03,72,3,1
	fcb	03,72,3,1
	fcb	03,72,7,1
	fcb	03,72,5,1
	fcb	03,72,3,1
	fcb	03,79,1,1
	fcb	03,79,3,1
	fcb	03,79,3,1
	fcb	03,79,3,1
	fcb	03,79,7,1
	fcb	06,12
	fcb	03,56,4,4
	fcb	00	; End of pattern

pattern_11
	; Voice 1: refrain music second pattern raised (double)
	fcb	06,11
	fcb	03,80,7,1
	fcb	03,79,7,1
	fcb	03,80,7,1
	fcb	03,79,7,3

	fcb	03,74,1,1
	fcb	03,74,3,1
	fcb	03,74,3,1
	fcb	03,74,3,1
	fcb	03,74,7,1

	fcb	06,13
	fcb	03,59,3,1
	fcb	03,56,1,1
	fcb	03,56,1,1
	fcb	00	; End of pattern

pattern_12
	; Voice 3: raised C-2 to measure
	fcb	06,01
	fcb	03,36,5,1
	fcb	03,48,1,1

	fcb	06,02
	fcb	03,58,3,1

	fcb	06,01
	fcb	03,36,3,1
	fcb	03,36,7,1

	fcb	06,02
	fcb	03,58,3,1

	fcb	06,01
	fcb	03,46,1,1
	fcb	03,48,1,1
	fcb	00	; End of pattern

pattern_13
	; Voice 3: raised 'C#2' measure
	fcb	06,01
	fcb	03,37,5,1
	fcb	03,49,1,1

	fcb	06,02
	fcb	03,58,3,1

	fcb	06,01
	fcb	03,37,3,1
	fcb	03,37,7,1

	fcb	06,02
	fcb	03,58,3,1

	fcb	06,01
	fcb	03,48,1,1
	fcb	03,49,1,1
	fcb	00	; End of pattern

pattern_14
	; Voice 3: 'G-1' measure
	fcb	06,01
	fcb	03,31,5,1
	fcb	03,43,1,1

	fcb	06,02
	fcb	03,58,3,1

	fcb	06,01
	fcb	03,31,3,1
	fcb	03,31,7,1

	fcb	06,02
	fcb	03,58,3,1

	fcb	06,01
	fcb	03,40,1,1
	fcb	03,40,1,1
	fcb	00	; End of pattern

pattern_15
	; Voice 2: first pattern raised +3 (double)
	fcb	06,10
	fcb	03,83,1,1
	fcb	03,83,1,1
	fcb	03,83,1,1
	fcb	03,83,1,1

	fcb	06,08
	fcb	03,67,3,1
	fcb	03,67,3,1
	fcb	03,68,5,1
	fcb	03,67,5,1
	fcb	03,65,3,1

	fcb	06,10
	fcb	03,83,1,1
	fcb	03,83,1,1
	fcb	03,83,1,1
	fcb	03,83,1,1

	fcb	06,08
	fcb	03,67,3,1
	fcb	03,67,3,1
	fcb	03,67,7,9
	fcb	00	; End of pattern

pattern_16
	; Voice 2: second pattern raised +3 (double)
	fcb	06,10
	fcb	03,83,1,1
	fcb	03,83,1,1
	fcb	03,83,1,1
	fcb	03,83,1,1

	fcb	06,08
	fcb	03,67,3,1
	fcb	03,67,3,1
	fcb	03,68,5,1
	fcb	03,67,5,1
	fcb	03,65,3,3

	fcb	03,67,1,1
	fcb	03,67,3,1
	fcb	03,67,3,1
	fcb	03,67,3,1
	fcb	03,67,7,9
	fcb	00	; End of pattern

pattern_17
	; Voice 1: frame 2305
	fcb	06,02	; snare drum
	fcb	03,62,3,1
	fcb	03,62,3,1

	fcb	06,11	; main instr.
	fcb	03,69,3,1
	fcb	03,69,3,1
	fcb	03,69,1,1
	fcb	03,69,1,1
	fcb	03,69,3,1
	fcb	03,71,3,1
	fcb	03,72,3,1
	fcb	03,74,1,1
	fcb	03,74,1,1
	fcb	03,74,3,1
	fcb	03,74,3,1
	fcb	03,74,3,1
	fcb	03,74,7,1

	fcb	06,12	; voice 1 end tom drum
	fcb	03,56,3,1

	fcb	06,11	; main instr.
	fcb	03,74,1,1
	fcb	03,76,1,1

	fcb	00	; End of pattern

pattern_18
	; Voice 1: frame 2497
	fcb	06,11	; main instr.
	fcb	03,77,1,1
	fcb	03,77,1,1
	fcb	03,76,3,1
	fcb	03,74,3,1
	fcb	03,72,3,1
	fcb	03,71,3,1
	fcb	03,69,3,1
	fcb	03,68,7,1

	fcb	06,02	; snare drum
	fcb	03,62,1,1

	fcb	06,11	; main instr.
	fcb	03,69,1,1
	fcb	03,69,3,1
	fcb	03,69,3,1
	fcb	03,71,3,1
	fcb	03,69,7,1

	fcb	06,12	; voice 1 end tom drum
	fcb	03,56,4,4

	fcb	00	; End of pattern

pattern_19
	; Voice 1: frame 2689
	fcb	06,08	; 'voice 2' snare-instrument
	fcb	03,72,1,1
	fcb	03,71,3,1
	fcb	03,70,1,1
	fcb	03,69,3,1

	fcb	03,72,1,1
	fcb	03,71,3,1
	fcb	03,70,1,1
	fcb	03,69,3,1

	fcb	03,72,1,1
	fcb	03,71,3,1
	fcb	03,70,1,1
	fcb	03,69,3,1

	fcb	03,72,1,1
	fcb	03,71,3,1
	fcb	03,70,1,1
	fcb	03,69,3,1

	fcb	03,72,1,1
	fcb	03,71,3,1
	fcb	03,69,1,1
	fcb	03,77,3,1
	fcb	03,76,3,1

	fcb	00	; End of pattern

pattern_20
	; Voice 1: frame 2881
	fcb	06,08	; 'voice 2' snare-instrument
	fcb	03,77,1,1
	fcb	03,76,3,1
	fcb	03,75,1,1
	fcb	03,74,3,1

	fcb	03,77,1,1
	fcb	03,76,3,1
	fcb	03,75,1,1
	fcb	03,74,3,1

	fcb	03,77,3,1
	fcb	03,76,3,1
	fcb	03,71,1,1
	fcb	03,70,3,1
	fcb	03,69,1,1
	fcb	03,68,3,1
	fcb	03,71,1,1
	fcb	03,70,3,1
	fcb	03,69,1,1
	fcb	03,68,3,1
	fcb	03,72,3,1
	fcb	03,71,3,1

	fcb	00	; End of pattern

pattern_21
	; Voice 1: frame 3457
	fcb	06,11	; main instr
	fcb	03,72,1,1
	fcb	03,71,3,1
	fcb	03,70,1,1
	fcb	03,69,3,1
	fcb	03,72,1,1
	fcb	03,71,3,1
	fcb	03,70,1,1
	fcb	03,69,3,1
	fcb	03,72,3,1
	fcb	03,74,3,1
	fcb	03,72,1,1
	fcb	03,71,3,1
	fcb	03,70,1,1
	fcb	03,69,3,1
	fcb	03,72,1,1
	fcb	03,71,3,1
	fcb	03,70,1,1
	fcb	03,69,3,1
	fcb	03,72,3,1
	fcb	03,74,3,1

	fcb	00	; End of pattern

pattern_22
	; Voice 1: frame 3841
	fcb	06,11	; main instr
	fcb	03,78,11,1

	; test pitch alteration
	;fcb	03,78,11,1	; this note is slurred down
	fdb	$0800,-(272/12)	; -272 total pitch bend during 12 frames
	fcb	03,78,4,0
	fdb	$0800,0		; stop pitch bend
	fcb	03,78,7,1
	fdb	$0707,0		; reset pitch alteration
	
	fcb	03,73,3,1
	fcb	03,71,3,1

	;fcb	03,71,7,1	; slurred
	fdb	$0800,-(134/12)	; -134 total pitch bend during 12 frames
	fcb	03,71,4,0
	fdb	$0800,0		; stop pitch bend
	fdb	$0707,0		; reset pitch alteration
	fcb	03,70,3,1

	fcb	03,70,3,1
	
	fcb	06,12	; voice 1 end tom drum
	fcb	03,56,3,1
	fcb	03,56,3,1
	
	fcb	06,11	; main instr
	fcb	03,70,1,1
	fcb	03,70,1,1
	fcb	03,71,3,1
	fcb	03,73,3,1
	
	fcb	00	; End of pattern

pattern_23
	; Voice 1: frame 4033
	fcb	06,11	; main instr
	fcb	03,73,11,1
	fcb	03,73,3,1
	fcb	03,76,5,1
	fcb	03,73,5,1
	
	;fcb	03,71,11,1	; slurred
	fdb	$0800,(202/15)	; 202 total pitch bend during 15 frames
	fcb	03,71,5,0
	fdb	$0800,0		; stop pitch bend
	fcb	03,71,6,1
	fdb	$0707,0		; reset pitch alteration
	
	;fcb	03,73,15,1	; slurred
	fdb	$0800,-(696/45)	; 696 total pitch bend during 45 frames
	fcb	03,73,15,1
	fdb	$0800,0		; stop pitch bend
	fdb	$0707,0		; reset pitch alteration
	
	fcb	06,12	; voice 1 end tom drum
	fcb	03,56,3,1
	fcb	06,12	; voice 1 end tom drum
	fcb	03,56,3,1
		
	fcb	00	; End of pattern

pattern_24
	; Voice 1: frame 4609
	fcb	06,11	; main instr
	fcb	03,74,7,1

	fcb	06,12	; voice 1 end tom drum
	fcb	03,56,3,1

	fcb	06,11	; main instr
	fcb	03,74,1,1
	fcb	03,74,1,1
	fcb	03,76,5,1
	fcb	03,74,5,1
	
	;fcb	03,72,11,1 ; slurred
	fdb	$0800,(87/12)	; 87 total pitch bend during 12 frames
	fcb	03,72,4,0
	fdb	$0800,0		; stop pitch bend
	fdb	$0707,0		; reset pitch alteration
	fcb	03,74,7,1

	fcb	06,12	; voice 1 end tom drum
	fcb	03,59,3,1
	;fcb	02,12,4,0,16,0,17,0,4,-1,4,-3,4,-4,4,-6,4,-8,4,-10,4,-12,4,-15,4,-18,4,-22,4,-28,4,-35 ; voice 1 'tom drum' 
	fcb	03,56,5,7 ; big tom: slurred
	;fcb	03,56,4,0 ; removed: this doesn't sound good
	;fcb	03,38,4,4
	
	fcb	03,59,3,1
	fcb	03,56,3,1
	
	fcb	00	; End of pattern

pattern_25
	; Voice 1: frame 4993
	fcb	06,11	; main instr
	fcb	03,76,7,1
	
	fcb	06,12	; voice 1 end tom drum
	fcb	03,56,3,1

	fcb	06,11	; main instr
	fcb	03,76,1,1
	fcb	03,76,1,1
	fcb	03,78,5,1
	fcb	03,76,5,1
	
	;fcb	03,74,11,1 ; slurred
	fdb	$0800,(87/12)	; 87 total pitch bend during 12 frames
	fcb	03,74,4,0
	fdb	$0800,0		; stop pitch bend
	fdb	$0707,0		; reset pitch alteration
	fcb	03,76,7,1
	
	fcb	06,12	; voice 1 end tom drum
	fcb	03,59,3,1
	fcb	03,56,5,7 ; slurred
	;fcb	03,56,4,0 ; removed: this doesn't sound good
	;fcb	03,38,4,4

	fcb	03,59,3,1
	fcb	03,56,3,1
	
	fcb	00	; End of pattern
	
pattern_26
	; Voice 1: frame 5185
	fcb	06,11	; main instr
	fcb	03,76,7,1
	
	fcb	06,12	; voice 1 end tom drum
	fcb	03,56,3,1

	fcb	06,11	; main instr
	fcb	03,76,1,1
	fcb	03,76,1,1
	fcb	03,78,5,1
	fcb	03,76,5,1
	
	;fcb	03,74,9,1 ; slurred
	fdb	$0800,(87/12)	; 87 total pitch bend during 12 frames
	fcb	03,74,4,0
	fdb	$0800,0		; stop pitch bend
	fdb	$0707,0		; reset pitch alteration
	fcb	03,76,5,1
	
	fcb	03,78,5,1
	fcb	03,80,3,1
	fcb	03,78,5,1
	fcb	03,80,5,1
	fcb	03,81,3,1

	fcb	00	; End of pattern
	
pattern_27
	; Voice 1: frame 5377 
	; NOTE: THIS IS A HALF-PATTERN (96 frames)
	fcb	06,08	; 'voice 2' snare-instrument
	fcb	03,82,1,1
	fcb	03,82,1,1
	fcb	03,82,1,1
	fcb	03,82,1,1
	fcb	03,82,1,1
	fcb	03,82,1,1
	fcb	03,80,1,1
	fcb	03,82,1,1
	fcb	03,82,1,1
	fcb	03,80,1,1
	fcb	03,82,3,1
	fcb	03,82,1,1
	fcb	03,82,1,1
	fcb	03,80,1,1
	fcb	03,80,1,1

	fcb	00	; End of pattern

pattern_28
	; Voice 1: frame 7297
	fcb	06,11	; main instr
	
	;fcb	03,67,39,1 ; slurred
	fcb	03,67,8,0
	fdb	$0800,(162/24)	; 162 total pitch bend during 24 frames
	fcb	03,67,8,0
	fdb	$0800,0		; stop pitch bend
	fcb	03,67,23,1
	fdb	$0707,0		; reset pitch alteration
	
	fcb	03,67,3,1
	fcb	03,69,3,1
	fcb	03,74,3,1
	fcb	03,72,3,1
	fcb	03,69,7,1
	
	fcb	00	; End of pattern
	
pattern_29
	; Voice 1: frame 7489
	fcb	06,11	; main instr
	;fcb	03,72,39,1 ; slurred
	fcb	03,72,8,0
	fdb	$0800,(215/24)	; 162 total pitch bend during 24 frames
	fcb	03,72,8,0
	fdb	$0800,0		; stop pitch bend
	fcb	03,72,23,1
	fdb	$0707,0		; reset pitch alteration
	
	fcb	03,74,3,1
	fcb	03,79,3,1
	fcb	03,78,3,1
	fcb	03,74,3,1
	fcb	03,69,7,1

	fcb	00	; End of pattern

pattern_30
	; Voice 1: frame 7681
	fcb	06,11	; main instr
	;fcb	03,67,39,1 ; slurred
	fcb	03,67,8,0
	fdb	$0800,(162/24)	; 162 total pitch bend during 24 frames
	fcb	03,67,8,0
	fdb	$0800,0		; stop pitch bend
	fcb	03,67,23,1
	fdb	$0707,0		; reset pitch alteration
	
	;fcb	03,75,11,1 ; slurred
	fdb	$0800,-(182/24)	; 162 total pitch bend during 24 frames
	fcb	03,75,8,0
	fdb	$0800,0		; stop pitch bend
	fdb	$0707,0		; reset pitch alteration
	fcb	03,74,3,1
	
	fcb	03,72,3,1
	fcb	03,69,7,1
	
	fcb	00	; End of pattern
	
pattern_31
	; Voice 1: frame 7873
	fcb	06,11	; main instr
	;fcb	03,74,39,1 ;slurred
	fcb	03,74,8,0
	fdb	$0800,-(176/24)	; total pitch bend during 24 frames
	fcb	03,74,8,0
	fdb	$0800,0		; stop pitch bend
	fcb	03,74,23,1
	fdb	$0707,0		; reset pitch alteration
	
	fcb	03,74,3,1
	fcb	03,76,3,1
	fcb	03,79,3,1
	fcb	03,78,3,1
	fcb	03,79,3,1
	fcb	03,81,3,1

	fcb	00	; End of pattern

pattern_32
	; Voice 1: frame 8065
	fcb	06,11	; main instr
	;fcb	03,79,23,1  ; slurred
	fcb	03,79,8,0
	fdb	$0800,(324/24)	; total pitch bend during 24 frames
	fcb	03,79,8,0
	fdb	$0800,0		; stop pitch bend
	fcb	03,79,7,1
	fdb	$0707,0		; reset pitch alteration
	
	fcb	03,81,1,1
	fcb	03,81,3,1
	fcb	03,81,1,1
	fcb	03,81,1,1
	fcb	03,81,3,1
	fcb	03,79,1,1
	fcb	03,81,3,1
	fcb	03,79,1,1
	fcb	03,78,3,1
	fcb	03,79,1,1
	fcb	03,78,3,1
	fcb	03,76,3,1
	fcb	03,74,3,1
	
	fcb	00	; End of pattern

pattern_33
	; Voice 1: frame 8257
	fcb	06,11	; main instr
	fcb	03,74,1,1
	fcb	03,74,3,1
	fcb	03,72,1,1
	fcb	03,74,3,1
	fcb	03,72,1,1
	fcb	03,71,3,1
	fcb	03,72,1,1
	fcb	03,71,3,1
	fcb	03,69,3,1
	fcb	03,67,3,1
	fcb	03,69,1,1
	fcb	03,69,3,1
	fcb	03,67,1,1
	fcb	03,69,3,1
	fcb	03,71,1,1
	fcb	03,72,3,1
	fcb	03,74,1,1
	fcb	03,76,3,1
	fcb	03,78,3,1
	fcb	03,79,3,1
	
	fcb	00	; End of pattern

pattern_34
	; Voice 1: frame 8449
	fcb	06,11	; main instr
	;fcb	03,83,39,1 ; slurred
	fcb	03,83,8,0
	fdb	$0800,-(218/24)	; total pitch bend during 24 frames
	fcb	03,83,8,0
	fdb	$0800,0		; stop pitch bend
	fdb	$0707,0		; reset pitch alteration
	fcb	03,81,23,1
	
	fcb	03,79,3,1
	fcb	03,81,3,1
	fcb	03,84,1,1
	fcb	03,84,1,1
	fcb	03,81,3,1
	fcb	03,86,1,1
	fcb	03,86,1,1
	fcb	03,84,3,1
	
	fcb	00	; End of pattern

pattern_35
	; Voice 1: frame 8641
	fcb	06,11	; main instr
	;fcb	03,88,47,3 ; slurred
	fcb	03,88,8,0
	fdb	$0800,-(483/24)	; total pitch bend during 24 frames
	fcb	03,88,8,0
	fdb	$0800,0		; stop pitch bend
	fcb	03,88,31,3
	fdb	$0707,0		; reset pitch alteration
	
	fcb	03,88,1,1
	fcb	03,88,1,1
	fcb	03,76,1,1
	fcb	03,89,1,1
	fcb	03,76,1,1
	fcb	03,84,1,1
	fcb	03,86,1,1
	
	fcb	00	; End of pattern

pattern_36
	; Voice 1: frame 8833,9025
	fcb	06,11	; main instr
	fcb	03,88,1,1
	fcb	03,88,1,1
	fcb	03,76,3,1
	fcb	03,86,3,1
	fcb	03,76,1,1
	fcb	03,84,3,1
	fcb	03,76,1,1
	fcb	03,83,3,1
	fcb	03,84,1,1
	fcb	03,76,1,1
	fcb	03,84,1,1
	fcb	03,86,1,1
	fcb	03,88,1,1
	fcb	03,88,1,1
	fcb	03,76,3,1
	fcb	03,86,3,1
	fcb	03,76,1,1
	fcb	03,84,3,1
	fcb	03,76,1,1
	fcb	03,83,3,1
	fcb	03,84,1,1
	fcb	03,76,1,1
	fcb	03,84,1,1
	fcb	03,86,1,1
	
	fcb	00	; End of pattern

pattern_37
	; Voice 1: frame 9217,9409
	fcb	06,11	; main instr
	fcb	03,88,1,1
	fcb	03,88,1,1
	fcb	03,76,3,1
	fcb	03,87,3,1
	fcb	03,76,1,1
	fcb	03,85,3,1
	fcb	03,76,1,1
	fcb	03,83,3,1
	fcb	03,85,1,1
	fcb	03,76,1,1
	fcb	03,85,1,1
	fcb	03,87,1,1
	fcb	03,88,1,1
	fcb	03,88,1,1
	fcb	03,76,3,1
	fcb	03,87,3,1
	fcb	03,76,1,1
	fcb	03,85,3,1
	fcb	03,76,1,1
	fcb	03,83,3,1
	fcb	03,85,1,1
	fcb	03,76,1,1
	fcb	03,85,1,1
	fcb	03,87,1,1

	fcb	00	; End of pattern

pattern_38
	; Voice 1: frame 9601,9793
	fcb	06,11	; main instr
	fcb	03,85,1,1
	fcb	03,85,1,1
	fcb	03,73,3,1
	fcb	03,83,3,1
	fcb	03,73,1,1
	fcb	03,82,3,1
	fcb	03,73,1,1
	fcb	03,80,3,1
	fcb	03,78,1,1
	fcb	03,73,1,1
	fcb	03,83,1,1
	fcb	03,85,1,1
	fcb	03,85,1,1
	fcb	03,85,1,1
	fcb	03,73,3,1
	fcb	03,83,3,1
	fcb	03,73,1,1
	fcb	03,82,3,1
	fcb	03,73,1,1
	fcb	03,80,3,1
	fcb	03,78,1,1
	fcb	03,73,1,1
	fcb	03,83,1,1
	fcb	03,85,1,1

	fcb	00	; End of pattern

pattern_39
	; voice 1 frame 10561
	fcb	06,14	; voice 1 song end drum
	fcb	03,63,7,17

	fcb	06,12	; voice 1 end tom drum
	fcb	03,56,3,1
	fcb	03,56,3,1
	fcb	00	; End of pattern

pattern_40
	; Voice 3: 'D-2' measure
	fcb	06,01
	fcb	03,38,5,1
	fcb	03,50,1,1

	fcb	06,02
	fcb	03,58,3,1

	fcb	06,01
	fcb	03,38,3,1
	fcb	03,38,7,1

	fcb	06,02
	fcb	03,58,3,1

	fcb	06,01
	fcb	03,48,1,1
	fcb	03,48,1,1
	fcb	00	; End of pattern

pattern_41
	; Voice 3: 'G-1' measure
	fcb	06,01
	fcb	03,28,5,1
	fcb	03,40,1,1

	fcb	06,02
	fcb	03,58,3,1

	fcb	06,01
	fcb	03,28,3,1
	fcb	03,28,7,1

	fcb	06,02
	fcb	03,58,3,1

	fcb	06,01
	fcb	03,38,1,1
	fcb	03,40,1,1
	fcb	00	; End of pattern

pattern_42
	; Voice 3: 'F#-1' measure
	fcb	06,01
	fcb	03,30,5,1
	fcb	03,42,1,1

	fcb	06,02
	fcb	03,58,3,1

	fcb	06,01
	fcb	03,30,3,1
	fcb	03,30,7,1

	fcb	06,02
	fcb	03,58,3,1

	fcb	06,01
	fcb	03,40,1,1
	fcb	03,42,1,1
	fcb	00	; End of pattern

pattern_43
	; Voice 3: frame 5377
	; taratatata
	fcb	06,08	; 'voice 2' snare-instrument
	fcb	03,51,1,1
	fcb	03,51,1,1
	fcb	03,51,1,1
	fcb	03,51,1,1

	fcb	06,12	; voice 1 end tom drum
	fcb	03,56,3,1

	fcb	06,08	; 'voice 2' snare-instrument
	fcb	03,49,1,1
	fcb	03,51,3,1
	fcb	03,49,1,1
	fcb	03,51,1,1
	fcb	03,51,1,1

	fcb	06,12	; voice 1 end tom drum
	fcb	03,59,3,1
	fcb	03,56,3,1
	fcb	00	; End of pattern

pattern_44
	; voice 3 frame 10561
	fcb	06,14	; voice 1 song end drum
	fcb	03,83,7,17

	fcb	06,12	; voice 1 end tom drum
	fcb	03,56,3,1
	fcb	03,56,3,1
	fcb	00	; End of pattern

pattern_45
	; Voice 2: frame 3457
	fcb	06,15
	fcb	03,100,3,1
	fcb	06,16
	fcb	03,93,3,1

	fcb	06,08
	fcb	03,69,3,1
	fcb	03,69,3,1
	fcb	03,69,5,1
	fcb	03,69,5,1
	fcb	03,67,5,1
	fcb	03,69,1,1
	fcb	03,69,3,1
	fcb	03,69,3,1
	fcb	03,67,3,1
	fcb	03,69,1,1
	fcb	03,67,1,1
	fcb	03,69,3,1

	fcb	06,15
	fcb	03,100,3,1
	fcb	06,16
	fcb	03,93,3,1
	fcb	00	; End of pattern

pattern_46
	; Voice 2: frame 3841
	fcb	06,15
	fcb	03,97,3,1
	fcb	06,16
	fcb	03,90,3,1

	fcb	06,08
	fcb	03,61,3,1
	fcb	03,61,3,1
	fcb	03,61,5,1
	fcb	03,61,5,1
	fcb	03,59,5,1
	fcb	03,61,1,1
	fcb	03,61,3,1
	fcb	03,61,3,1
	fcb	03,59,3,1
	fcb	03,61,1,1
	fcb	03,59,1,1
	fcb	03,61,3,1

	fcb	06,15
	fcb	03,97,3,1
	fcb	06,16
	fcb	03,90,3,1
	fcb	00	; End of pattern

pattern_47
	; Voice 2: frame 4609
	fcb	06,15
	fcb	03,105,3,1
	fcb	06,16
	fcb	03,98,3,1

	fcb	06,08
	fcb	03,62,3,1
	fcb	03,62,3,1
	fcb	03,62,5,1
	fcb	03,62,5,1
	fcb	03,60,5,1
	fcb	03,62,1,1
	fcb	03,62,3,1
	fcb	03,62,3,1
	fcb	03,60,3,1
	fcb	03,62,1,1
	fcb	03,60,1,1
	fcb	03,62,3,1

	fcb	06,15
	fcb	03,105,3,1
	fcb	06,16
	fcb	03,98,3,1
	fcb	00	; End of pattern

pattern_48
	; Voice 2: frame 4993
	fcb	06,15
	fcb	03,107,3,1
	fcb	06,16
	fcb	03,100,3,1

	fcb	06,08
	fcb	03,64,3,1
	fcb	03,64,3,1
	fcb	03,64,5,1
	fcb	03,64,5,1
	fcb	03,62,5,1
	fcb	03,64,1,1
	fcb	03,64,3,1
	fcb	03,64,3,1
	fcb	03,62,3,1
	fcb	03,64,1,1
	fcb	03,62,1,1
	fcb	03,64,3,1

	fcb	06,15
	fcb	03,107,3,1
	fcb	06,16
	fcb	03,100,3,1
	fcb	00	; End of pattern

pattern_49
	; Voice 2: frame 5377
	; NOTE: THIS IS A HALF-PATTERN (96 frames)
	fcb	06,08
	fcb	03,79,1,1
	fcb	03,79,1,1
	fcb	03,79,1,1
	fcb	03,79,1,1
	fcb	03,79,1,1
	fcb	03,79,1,1
	fcb	03,77,1,1
	fcb	03,79,1,1
	fcb	03,79,1,1
	fcb	03,77,1,1
	fcb	03,79,3,1
	fcb	03,79,1,1
	fcb	03,79,1,1
	fcb	03,77,1,1
	fcb	03,77,1,1

	fcb	00	; End of pattern

pattern_50
	; voice 2 frame 10561
	fcb	06,14	; voice 1 song end drum
	fcb	03,83,7,17

	fcb	06,12	; voice 1 end tom drum
	fcb	03,56,3,1
	fcb	03,56,3,1
	fcb	00	; End of pattern

patterns_end


; all values below this point are auto-generated at runtime

;reserve room for 32 wavetable contains 15 waveform 16-bit pointers from waveform_lookup + pitch alteration, (60 bytes each)
wavetable	rmb	1920

noise2048	rmb	2048

sin256		rmb	256
saw256		rmb	256
triangle256	rmb	256
mute256		rmb	256

sin256_s1	rmb	256	; volume shifted by 1
sin256_s2	rmb	256	; volume shifted by 2

saw256_s1	rmb	256	; volume shifted by 1
saw256_s2	rmb	256	; volume shifted by 2

triangle256_s1	rmb	256	; volume shifted by 1
triangle256_s2	rmb	256	; volume shifted by 2

square256	rmb	256
square256_s1	rmb	256	; volume shifted by 1
square256_s2	rmb	256	; volume shifted by 2

rect256		rmb	256	; same as square but inverted to help with mixing
rect256_s1	rmb	256
rect256_s2	rmb	256

*********************
* End

	
	END start
